"""
Association and interaction testing module for PANAMA
(1) association testing
(2) interaction testing
(3) parallelization using ipython
"""

import sys, os, string, cPickle, copy, pdb
sys.path.append('./../../pygp')
sys.path.append('./../../')
#gplvm model
from pygp.covar import linear, noise, combinators, fixed, mu
import pygp.gp.gplvm as GPLVM
# customized covariance function:
from interaction_covar import *
import pygp.priors.lnpriors as lnpriors
import linear_regression as linreg
import logging as LG
import scipy as SP
import numpy as N
from panama.utilities import fdr
import scipy.stats as st
import copy
import testing_core

try:
    from IPython.kernel import client
    ipy_version = 0.10
except ImportError:
    from IPython.parallel import Client as client
    ipy_version = 0.12

tc_clients = None

def interface(S, P, K, covs=None, I = None, model = 'LMM', return_fields= ['pv'], parallel = False,
	      add_mean = True, jobs = 1, file_directory=None, delta_opt_params=None, Ftest = False):
    
    """
    interface function for association testing
    S: SNPS [indiv X snps]
    Y: expr [indic X genes]
    covs: covariates 
    model: model to use (LMM/LINREG)
    return_filed: fields to caculate and return
    file_directory: if provided data handling and result storing will be handeled via file IO
    delta_opt_params: parameters for delta optimization 
    """

    #1. check dimensions
    [Nn, Np] = P.shape
    Ns       = S.shape[1]

    #check input arguments
    assert (S.shape[0]==Nn), 'dimension error'
    if K is not None:
        assert (K.shape[0]==Nn), 'dimension error'
        assert (K.shape[1]==Nn), 'dimension error'
    if I is not None:
        assert (I.shape[0]==Nn), 'dimension error'
    #covariates: 
    if covs is None:
	covs = SP.zeros([Nn,0])
        if add_mean:
            #add mean column ( we should generally do this )
            covs = SP.concatenate((covs,SP.ones([Nn,1])),axis=1)

    #figure out base path
    cwd=os.getcwd()
    base_path      = os.path.dirname(__file__)
    glob_base_path = os.path.join(cwd,base_path)
    paths = [glob_base_path]

    if file_directory is not None:
        if not os.path.exists(file_directory):
            print "file mode for testing specified but directory %s not existing; disableing" % file_directory

    #check whether to use file mode
    file_mode = ((file_directory is not None) and os.path.exists(file_directory))
    #setup file handling if switched on
    in_file = None
    in_file_path = None
    out_file = None
    out_file_base = None
    if file_mode:
        import h5py
        in_file = os.path.join(file_directory,'data.hdf5')
	in_file_path = os.path.abspath(in_file)
        out_file_base = os.path.join(file_directory,'result_')
        #store matrix objects
        _data = {'S':S,'P':P,'K':K,'I':I,'covs': covs}
        f = h5py.File(in_file,'w')
        for key in _data:
            if _data[key] is None:
                continue
            f.create_dataset(name=key,data=_data[key],chunks=True)
        f.close()
        del(f)

    if not parallel:
        RV = testing_core.wrapper(S, P, K, covs = covs, I = I, model = model,
				  path_append = paths,index_s = SP.arange(S.shape[1]),
				  index_p = SP.arange(P.shape[1]),
				  in_file = in_file_path,
				  delta_opt_params=delta_opt_params, Ftest = Ftest)        
    else:
	# setup engines and perform checks
	tc, eng_ids = start_engines(jobs)

	# split the data according to the number of jobs
        #note jobs is the max. number of jobs...
	Y_split = split_jobs(P, jobs)

	ipython_jobs = []
	for n in xrange(len(Y_split)):
	    i0 = Y_split[n][0]
	    i1 = Y_split[n][1]
	    P_n = Y_split[n][2]

            #arguments of job
            #convert all paths to absolute paths
            xargs=SP.array([S, P_n, K, covs, I, range(i0, i1), range(S.shape[1]),
			    model, return_fields, paths, out_file, in_file_path,
			    Ftest, delta_opt_params],dtype='object')
            #if in file_mode, overwrite:
            if file_mode:
                out_file = '%s%03d.pickle' % (out_file_base,n)
                if os.path.exists(out_file):
                    os.remove(out_file)
                xargs[0:4] = None
                xargs[-3]  = os.path.abspath(out_file)

	    if ipy_version == 0.10:
		job = client.MapTask(testing_core.wrapper, xargs.tolist())
		ipython_jobs.append(tc.run(job))
	    else:
		dview = eng_ids[:]
		lbview = tc.load_balanced_view()
		job = lbview.apply(testing_core.wrapper, *xargs.tolist())
		ipython_jobs.append(job)
	    
        #wait for job completion:
	if ipy_version == 0.10:
	    tc.barrier(ipython_jobs)
	else:
	    tc.wait(ipython_jobs)

	RV = None
	for n in xrange(len(Y_split)):
	    i0 = Y_split[n][0]
	    i1 = Y_split[n][1]

            #fecht results from job
	    if ipy_version == 0.10:
		_rv = tc.get_task_result(ipython_jobs[n])
	    else:
		_rv = ipython_jobs[n].get()
	    
            if isinstance(_rv,str):
		f = open(_rv,'rb')
                _rv = cPickle.load(f)
		f.close()
                _rv = testing_core.filter_dict(_rv,return_fields)

            #do we need to create result structures?
            if RV is None:
                RV = {}
                #get shape of all fields we need
                #append dimensions form result structure
                for key in _rv.keys():
                    val = _rv[key]
                    dim0 = [Ns,Np]
                    dim0.extend(list(val.shape[2::]))
                    RV[key] = SP.empty(dim0)
            #slot in result fileds
            for key in _rv.keys():
                RV[key][:,i0:i1] = _rv[key]
    return [RV[key] for key in return_fields]


#### internal helper functions ####


 
        

def split_jobs(Y, Njobs):
    #slit phenotype matrix into jobs
    #think about splitting snps also
    splits = []


    [N, Np] = Y.shape
    #maximal splitting range is one job per phenotype
    Njobs = min(Njobs,Np)

    #figure out phenotypes per job (down rounded)
    npj   = int(SP.floor(SP.double(Np)/Njobs))
    
    i0 = 0
    i1 = npj
    for n in xrange(Njobs):
        if n==(Njobs-1):
            #make sure last jobs spans all the rest.
            i1 = Np
        Y_ = Y[:,i0:i1]
	splits.append([i0, i1, Y_])
        #nex split
        i0 = i1
        i1 = i1 + npj
        
    return splits

def start_engines(jobs):
    """
    Starts the engines and performs some checks

    jobs: int
    """

    global tc_clients
    
    if ipy_version == 0.10:
	tc = client.TaskClient()
	mec = client.MultiEngineClient()
	eng_ids = mec.get_ids()
    else:
	if tc_clients == None:
	    tc = client()
	    eng_ids = tc.ids
	    tc_clients = tc
	else:
	    tc = tc_clients
	    eng_ids = tc.ids
	
    #reset engines
    #mec.reset()
    assert eng_ids > 0, "No engine started"	
    #assert eng_ids > jobs, "Not enough engines for the number of jobs required"

    return tc, eng_ids

                




import os, sys, cPickle, pdb
sys.path.append('./../../pygp/')
sys.path.append("./../../")
import linear_regression as linreg
import scipy as SP
import numpy as N
import numpy.random as random
import scipy.stats as st
from panama.utilities import mkl
from panama.utilities import fdr
from panama.utilities import io_wrapper as pw
from panama.core.gpasso_joint import *
import logging as LG

try:
    from panama.tests.other_models import sva
    from panama.tests.other_models import ice    
    import panama.tests.other_models.peer.vbfa as VBFA
except ImportError:
    pass

from panama.utilities.plot_parameters import *
file_type = "hdf5"


def save_everything(qv, pv, dir_name, name):
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    pw.write((qv, pv), os.path.join(dir_name,name), file_type)


def PANAMA(expr, snps, model = "default", tf_correction = False, pop_struct = False,
	   write_files = False, dump_asso_object = False, verbose_plots = False,
	   dir_name = "./", statistics = False, parallel = True,
	   jobs = 2, return_asso_object = False, max_train_iter = 10, **kw_args):

    if model == "default":
	panama_model = "gplvm"
    else:
	panama_model = "gplvm_interact"

    if dir_name != None and verbose_plots:
	verbose_dir = os.path.join(dir_name,'verbose_plots')
	if not os.path.exists(verbose_dir):
	    os.makedirs(verbose_dir)
    else:
	verbose_dir = None
	
    Kpop = None
    if type(pop_struct) != type(True): # if it's not a boolean
	assert pop_struct.shape[0] == pop_struct.shape[1], "Kpop must be NxN"
	assert pop_struct.shape[0] == expr.shape[0], "Kpop must be NxN, Y should be NxD"	
	
	Kpop = pop_struct # use the matrix
	pop_struct = True # this is needed (see assignment pop_cf = pop_struct)	
    elif pop_struct: # if it's true but no Kpop is provided, compute it
	Kpop = SP.cov(snps)
     

    asso = GPLVMassoJ(gplvm_init = "randn",
		      model = panama_model,
		      mean_cf = True,
		      pop_cf = pop_struct,
                      asso_method = 'LMM',
                      parallelize= parallel,
		      ard = True,
                      Njobs = jobs,
		      verbose_plot_dir=verbose_dir,
		      Kpop = Kpop,
                      **kw_args
                      )

    asso.add_interactions = False # TEMPORARY
    asso.setData(snps=snps,pheno=expr,normalize_geno=False, normalize_pheno = False)
    asso.train(Niter=max_train_iter,maxiter=1E5,Nrestarts=1,gradcheck=False, diagnostics=False)
    
    if dump_asso_object:
	if not os.path.exists(dir_name):
	    os.makedirs(dir_name)
	# backend is "pickle" because hdf5 can't serialize objects
	pw.write(asso, os.path.join(dir_name,"asso_object"), "pickle")
	objects_dir = os.path.join(dir_name,"objects/")
	pw.write(asso.getLatent(ard_reweight=True,
				pca_rotation=True,
				rotate_varimax = False),
		 os.path.join(objects_dir,"PANAMA_factors"), file_type)

    
    if parallel:
	# Don't use multithreaded scipy, we are going to parallelize and we need the cores    
	mkl.set_num_threads(1)

    qv, pv = asso.getAssociations()
    # qv_inter, pv_inter = asso.getInteractions()

    if statistics:
	LG.info("Associations: %d" % (qv<0.05).sum())
	gc = fdr.estimate_lambda(pv)
	LG.info("Genomic control: %.4f" % (gc))

    if tf_correction:
	# set to None to force recalc
	asso.QV_asso = None
	qvT, pvT = asso.getAssociations(correct_trans=True)
	if statistics:
	    gcT = fdr.estimate_lambda(pvT)
	    LG.info("associations: %d" % (qvT<0.05).sum())
	    LG.info("genomic control: %.4f" % (gcT))

    if write_files:
	save_everything(qv, pv, dir_name, "PANAMA")
	if tf_correction:
	    save_everything(qvT, pvT, dir_name, "PANAMA_trans")

    # restore default
    mkl.set_num_threads(mkl.get_num_cores())
    
    if tf_correction:
	return qv, pv, qvT, pvT

    if return_asso_object:
	return qv, pv, asso

    return qv, pv


def LIMMI(expr, snps, dir_name = None, write_files = True,
	  statistics = False, max_interaction_iter = 5, dump_asso_object = False, **kw_args):
    
    if dir_name is not None:
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)

    qv, pv, asso = PANAMA(expr, snps, model = "gplvm_interact", return_asso_object = True,
			  dump_asso_object = False,  dir_name = dir_name, write_files = True, **kw_args)

    asso.get_explained_var()
    
    objects_dir = os.path.join(dir_name,"objects/")

    if not os.path.exists(objects_dir):
	os.makedirs(objects_dir)

    if dump_asso_object:
	pw.write(asso, os.path.join(objects_dir ,"PANAMA_object"), "pickle")

    asso.add_interactions = True
    asso.add_associations = False
    num_interactions = None
    
    for i in range(max_interaction_iter):
	# train LIMMI
	asso.train(Niter_interact = 1, maxiter=1E5, Nrestarts=1, gradcheck=False, diagnostics=False)
	asso.get_explained_var()
	# run LMM scan
	qv, pv = asso.getAssociations()	
	# save associations
	save_everything(qv, pv, dir_name, "LIMMI_iteration%d" % i)
	# save interactions and factors
	pw.write(asso.interactions_qv, os.path.join(objects_dir,"LIMMI_iteration%d_qv" % i), file_type)
	pw.write(asso.getLatent(ard_reweight=True,
				pca_rotation=True,
				rotate_varimax = False),
		 os.path.join(objects_dir,"LIMMI_iteration%d_factors" % i), file_type)
	pw.write(asso.interactions_pv, os.path.join(objects_dir ,"LIMMI_interaction_pv"), file_type)
	
	if dump_asso_object:
	    pw.write(asso, os.path.join(objects_dir,"LIMMI_iteration%d_object % i"), "pickle")
	
	if num_interactions != None and num_interactions == len(asso.candidate_interactions):
	    break
	else:
	    num_interactions = len(asso.candidate_interactions)
	
    pass

def linear(expr, snps, write_files = False, dir_name = None, statistics = False, parallel = False, jobs = 0):
    LG.info("Testing LINEAR:")
    lods = linreg.run_associations(expr,snps,None)
    pv = (1. - st.chi2.cdf(2*lods, 1))
    qv = fdr.qvalues(pv)

    if statistics:
	print "associations: %d" % (qv<0.05).sum()
	gc = fdr.estimate_lambda(pv)
	print "genomic control: %.4f" % (gc)

    if write_files:
	save_everything(qv, pv, dir_name, "LINEAR")

    return qv, pv

def SVA(expr, snps, interactions = True, write_files = False, dir_name = None,
	parallel = True, jobs = 10, statistics = False, proper = False, sparse_pvmin = 1.0,
	Kpop = None):
    
    LG.info("Testing SVA:")

    if proper:
	Y, X, W = sva.sva(expr, covariates = snps, return_ZN = True)
	lods = linreg.run_associations(Y,snps,None)
	pv = (1. - st.chi2.cdf(2*lods, 1))
	qv = fdr.qvalues(pv)
    else:
	qv, pv, X, W, Y = sva.run_eqtl(expr, snps)
	
    if statistics:
	print "associations: %d" % (qv<0.05).sum()
	gc = fdr.estimate_lambda(pv)
	print "genomic control: %.4f" % (gc)

    if interactions:
	QVMi, PVi = model_free_interactions(Y, snps, X, Kpop, parallel, jobs, sparse_pvmin)
	interactions_intermediate_qv = QVMi
	
	if write_files:
	    objects_dir = os.path.join(dir_name,"objects/")
	    pw.write(interactions_intermediate_qv, os.path.join(objects_dir ,"SVA_qv"), file_type)
	    pw.write(X, os.path.join(objects_dir ,"SVA_factors"), file_type)
	    pw.write(PVi, os.path.join(objects_dir, "SVA_interactions_pv"), file_type)	    

    if write_files:
	save_everything(qv, pv, dir_name, "SVA")


    return qv, pv


def ICE(expr, snps, write_files = False, statistics = False, parallel = False, jobs = 0, dir_name = None):
    def split(n, iterable, padvalue=None):
	"split(3, 'abcdefg', 'x') --> ('a','b','c'), ('d','e','f'), ('g','x','x')"
	return izip_longest(*[iter(iterable)]*n, fillvalue=padvalue)

    if parallel:
	from multiprocessing import Process, Pool
	from itertools import izip_longest
	Q = snps.shape[1]

	splits = list(split(Q/jobs, range(Q-(Q%jobs)), padvalue=None))
	if Q % jobs != 0:
	    splits.append(range(Q-(Q%jobs), Q))

	pool = Pool(processes=len(splits))
	proc = []
	for s in splits:
	    proc.append(pool.apply_async(ice.run_eqtl, (expr, snps[:,s])))

	qv, pv = [], []
	for p in proc:
	    qvi, pvi = p.get()
	    qv.append(qvi)
	    pv.append(pvi)

	qvs = qv[0]
	for i in range(1,len(qv)):
	    qvs = SP.concatenate((qvs, qv[i]))

	pvs = pv[0]
	for i in range(1,len(pv)):
	    pvs = SP.concatenate((pvs, pv[i]))

	qv, pv = qvs, pvs
	    
    else:
	print "Testing ICE:"
	qv, pv = ice.run_eqtl(expr, snps)

    if statistics:
	print "nrAsso: %d" % (qv<0.05).sum()
	gc = fdr.estimate_lambda(pv)
	print "genomic control: %.4f" % (gc)
    if write_files:
	save_everything(qv, pv, dir_name, "ICE")

    return qv, pv

def PEER(expr, snps, write_files = False, dir_name = None, statistics = False, interactions = True, sparse_pvmin = 1.0, **kw_args):
    prior = 1

    if prior==1:
        priors = {'Alpha':{'priors':[1E-2,1E2]}}
    elif prior==2:
        priors = {'Alpha':{'priors':[1E-1,1E1]}}
    elif prior==3:
        priors = {'Alpha':{'priors':[1E-2,1E4]}}


    #use prior settings from peer publication:
    Ng = expr.shape[1]
    #use a fixed setting for the number of gnes
    # Ng = 6000.

    #priors = {'Alpha':{'priors':[1E-7*Ng,1E-1*Ng]},'Eps':{'priors':[1.,100.]}}
    #priors = {'Alpha':{'priors':[1E-6*Ng,1E-1*Ng]},'Eps':{'priors':[1.,10.]}}
    nIterations = 3
    components = 50

    Y = expr
    vbfa = VBFA.CVBFA(E1=Y,priors=priors,nIterations=nIterations,components=components)
    vbfa.iterate()
    Yp = vbfa.getPrediction().E1

    Ycorr = Y-Yp
    #run linear
    lods = linreg.run_associations(Ycorr,snps,None)
    pv = (1. - st.chi2.cdf(2*lods, 1))
    qv = fdr.qvalues(pv)

    if interactions:
	QVMi = model_free_interactions(expr, snps, vbfa.S.E1, False, 0, sparse_pvmin)
	interactions_intermediate_qv = QVMi
	
	if write_files:
	    objects_dir = os.path.join(dir_name,"objects/")
	    pw.write((interactions_intermediate_qv, vbfa.S.E1), os.path.join(objects_dir ,"PEER_object"), file_type)

    
    if statistics:
	print "associations: %d" % (qv<0.05).sum()
	gc = fdr.estimate_lambda(pv)
	print "genomic control: %.4f" % (gc)

    if write_files:
	save_everything(qv, pv, dir_name, "PEER")

    return qv, pv


def model_free_interactions(expr, snps, X, Kpop, parallel, jobs, pv_min):
    sparse_storage = True

    K = SP.zeros((expr.shape[0],expr.shape[0]))

    if Kpop != None:
	K = Kpop

    PV = []
    INDEX = []

    n_effective_tests = X.shape[1] * expr.shape[1] * snps.shape[1]

    # sometimes (like when we are simulating) we don't want to store sparse QVs, so
    # let's just compute the whole thing
    if not sparse_storage:
	PV = SP.empty((self.snps.shape[1], Y.shape[1],X.shape[1]))

    pv_def = None
    
    for i in range(X.shape[1]):
	[lod, pv_lmm] = testing.interface(snps, expr, K, I = X[:,i:i+1], model = "LMM",
				   return_fields = ['lod', 'pv'],
				   parallel = parallel, jobs = jobs)

	pv_lmm = st.chi2.sf(2*(lod),1)

	if i == 0:
	    pv_def = pv_lmm.copy()
	    
	if sparse_storage:
	    #1. figure out which pv are < min
	    Irel = pv_lmm<pv_min
	    pv_rel = pv_lmm[Irel]
	    index = SP.nonzero(Irel)
	    index_snps = index[0]
	    index_genes  = index[1]
	    index_factors = SP.ones_like(index_genes)*i
	    index_full = SP.concatenate((index_snps[:,SP.newaxis],index_genes[:,SP.newaxis],index_factors[:,SP.newaxis]),axis=1)
	    INDEX.extend(index_full)
	    PV.extend(pv_rel)
	else:
	    PV[:,:,i] = pv_lmm

    if sparse_storage:
	#convert everything into arrays
	PV = SP.array(PV)
	INDEX = SP.array(INDEX)
	QV = fdr.qvalues(PV, m = n_effective_tests, fix_lambda = pv_min)
	QVMi = SP.concatenate((SP.array(INDEX), SP.array(QV)[:, SP.newaxis]), axis=1)
    else:
	QV = fdr.qvalues(PV, m = n_effective_tests)
	QVMi = QV.min(axis=2) # minimize across factors

    return QVMi, pv_def



import cPickle
import scipy as SP

def load(filename, file_type = "pickle"):

    if file_type == "pickle":
	f = open(filename + ".pickle", "r")
	contents = cPickle.load(f)
	f.close()
    else:
	import h5py
	f = h5py.File(filename + ".h5",'r')
	contents = SP.asarray(f['data'])
	f.close()
	
    return contents

def write(data, filename, file_type = "pickle"):

    if file_type == "pickle":
	f = open(filename + ".pickle", "w")
	cPickle.dump(data, f, -1)
	f.close()
    else:
	import h5py
	f = h5py.File(filename + ".h5", 'w')
	f.create_dataset(name="data", data=data)
	f.close()


    return data



import scipy as SP
# import lmm and whatever is needed
import logging as LG
import scipy as SP
import cPickle
import os
import pdb

#LOAD LMM
try:
    #import panama.core.lmm.lmm as LMM
    import lmm.lmm as LMM
except ImportError:
    LG.error("Fast C++ LMM not available, falling back to python version")
    import lmm.lmm_fast as LMM

delta_opt_params_default={'numintervalsAlt':0,
                    'ldeltaminAlt':-1.0, 'ldeltamaxAlt':1.0,
                    'numintervals0':100,
                    'ldeltamin0':-5.0, 'ldeltamax0':5.0}


def hdf5_load_data(in_file,index_s,index_p):
    """
    loads data from hdf5 file
    """
    import h5py
    # read arguments from hdf5 instead of assuming they are passed form local memory
    f = h5py.File(in_file,'r')
    S = SP.array(f['S'][:,index_s])
    P = SP.array(f['P'][:,index_p])
    K = SP.array(f['K'])
    covs = SP.array(f['covs'])
    I = None
    if 'I' in f.keys():
        I = SP.array(f['I'])
    return S, P, K,covs,I


def filter_dict(D,keys):
    """
    filter_dict
    """
    for key in D.keys():
        if key not in keys:
            del(D[key])
    return D


def wrapper(S, P, K, covs = None, I = None, index_p=None, index_s=None, model = 'LMM',
	    return_fields= ['pv'], path_append=[], out_file= None, in_file=None,
	    Ftest = True, delta_opt_params=None):
    """wrapper"""
    
    import sys
    sys.path.extend(path_append)
    from testing_core import *

    if delta_opt_params is None:
        delta_opt_params = delta_opt_params_default

    #load hdf5 data if specified
    if in_file is not None:
        S, P, K,covs,I = hdf5_load_data(in_file,index_s,index_p)
        
    # load existing dump file if available 
    if (out_file is not None) and os.path.exists(out_file):
	f = open(out_file,'rb')
        rv = cPickle.load(f)
	f.close()
        filter_dict(rv,return_fields)
        return rv

    #return value is a hash of results that depends on actual method
    rv = {}
    if model=='LMM':
        if I is not None:
            lod,pval =  LMM.train_interactions(S, P, K, covs, I, use_ftest = Ftest, **delta_opt_params)
        else:
	    lod, pval =  LMM.train_associations(S, P, K, covs, **delta_opt_params)

        lod = lod.T
        pval = pval.T
    else:
        # ML
        if I is not None:
            print "not implemented!"
            pass
        else:
	    lod = linreg.run_associations(P, S, None)
	    pval = (1. - st.chi2.cdf(2*lod, 1))
    rv['index_p'] = index_p
    rv['index_s'] = index_s 
    rv['lod'] = lod
    rv['pv'] = pval
    # dump to file if specified
    if out_file is not None:
	f = open(out_file,'wb')
        cPickle.dump(rv,f,-1)
	f.close()
        return out_file
    else:
        #return 
        filter_dict(rv,return_fields)
        return rv



import os, sys, pdb, numpy
import distutils.core
from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext

# DEFINE VERSION HERE
version_number = '0.3.7'
compile_flags = ["-O3"]

if sys.platform != "darwin":
    compile_flags.append("-march=native")
else:
    os.environ['ARCHFLAGS'] = "-arch i386 -arch x86_64"

ext_modules = [Extension(
    name="panama.core.lmm.lmm",
    language="c++", # mac g++ -dynamic != gcc -dynamic
    sources=["panama/core/lmm/lmm.pyx",
	     "panama/core/lmm/c_lmm.cpp",
	     "panama/core/lmm/utils/Gamma.cpp",
	     "panama/core/lmm/utils/FisherF.cpp",
	     "panama/core/lmm/utils/Beta.cpp",
	     "panama/core/lmm/utils/MathFunctions.cpp",
	     "panama/core/lmm/utils/BrentC.cpp"
             ],
    include_dirs = [numpy.get_include()],
    extra_compile_args = compile_flags,
    )]


setup(name = 'panama',
      url = 'http://ml.sheffield.ac.uk/qtl/panama/',
      version = version_number,
      description = 'a novel probabilistic model to account for confounding factors in eQTL studies',
      long_description = open('README.txt').read(),
      author = 'Nicolo Fusi*, Oliver Stegle*, Neil Lawrence',
      author_email = 'nicolo.fusi@sheffield.ac.uk',
      packages = ['panama.core', 'panama.core.lmm', 'panama.utilities', 'panama.data'],
      package_data = {'': ['*.csv', '*.pdf', '*.txt']},
      scripts = ['bin/panama'],
      py_modules = ['panama.__init__'],      
      cmdclass = {'build_ext': build_ext, 'inplace': True},
      install_requires = ['numpy >= 1.5',
			  'scipy >= 0.8',
			  'matplotlib >= 0.99',
			  'ipython >= 0.10',
			  'pyzmq',
			  'Cython',
			  'pandas',
			  'pygp'],
      license = 'GPL v2.0',
      ext_modules = ext_modules
      )
